{% for header in license_headers %}
# {{ header }}
{% endfor %}

from __future__ import annotations

import {{ config.unname }}


def test_placeholder():
    """
    This is a placeholder test.
    Write actual tests and delete this one
    """
    assert {{ config.unname }}.__version__

