{% for header in license_headers %}
# {{ header }}
{% endfor %}
---

image: fedora/latest
packages:
  - nox
  - python3.8
  - python3.9
  - python3.10-devel
  - python3.11-devel
  - python3.12-devel
sources:
  - https://git.sr.ht/~gotmax23/{{ config.name }}
secrets: []
  # gotmax23/{{ config.name }}-dev webhook
  # -
tasks:
  - copr-webhook: |
      cd {{ config.name }}
      nox -e releaserr -- \
        copr-webhook ~/.copr-dev-hook --branch origin/main --skip-if-missing
  - nox-test: |
      cd {{ config.name }}
      nox -v -e test
  - nox-lint: |
      cd {{ config.name }}
      nox -v -e lint

{#
# vim: ft=yaml.jinja2:
#}
